/***********************************************************************
Class name:       Project1
Description:      infinite loop that prints a prompt, reads a line,
parses it (breaks it up into its constituent commands), starts a 
new thread to handle each of the different commands, and then waits for 
all the threads to finish before printing the next prompt.
Author:           Fok, Data Chun	
Student ID:       9018031113
Lecture:          CS 537-1
Other class :     Command, StdOut, StdErr
     
***********************************************************************/

import java.io.*;
import java.util.*;

//project 1 

class Project1{ 

	//the main class
	public static void main(String[] args) {
		
 		BufferedReader stdin = new BufferedReader( new InputStreamReader(System.in));
		
		String s = ""; //the input command from the user

		System.out.println("Welcome to my shell");
		
		//infinite loop for the input command
		for(;;){
		
			try{
			    System.out.print(">");
			    s = stdin.readLine();//get the input string
			    if (s==null){ //eof detected terminate
				System.exit(0);
			    }
			}
			catch(IOException e){
				e.printStackTrace();
			}
			
			//parse the string in to commands
			StringTokenizer token = new StringTokenizer(s,"&");
			Thread[] t = new Thread[token.countTokens()];
			int i,cnt;//loop counter
			String[] cmds = new String [token.countTokens()];
			
			//create a thread for each command
			i=0; 
			while (token.hasMoreTokens()){
			    cmds[i] = token.nextToken();
			    t[i] = new Thread(new Command(cmds[i]));
			    t[i].start();
			    i++;
			}
			
			//wait for all thread finish
			try{
			    for (cnt=0;cnt<i;cnt++){
				t[cnt].join();
			    }
			}
			catch(InterruptedException e){
			    System.out.println("INTERRUPTED EXCEPTION DETECTED in running the command!");
			    e.printStackTrace();
			}
			
		}
	}
}

//Command class
//handle filtering of space and tab and detection of "exit"
//create thread for the output streams
class Command implements Runnable{
		
    private String cmdline;
    public Process pr;		
    
    //contructor
    public Command(String cmdline){
	this.cmdline = cmdline;
    }
		
    public void run(){			
	
	try{
	    //filter out all space or tab
	    StringTokenizer st=new StringTokenizer(cmdline);
	    if(st.countTokens()>=1){
		    // if the command is exit, quit theshell
		    if (st.nextToken().equals("exit")){
			System.exit(0);
		    }
	    
		    Runtime rt = Runtime.getRuntime();
		    pr = rt.exec(cmdline);
	    }
	    else{
		pr = null; //do not create process if command is space or tab
	    }
	}
	catch(IOException e){
	    System.out.println("IOexception: " +cmdline+ ": command not found");
	}
	
	//check if a process is create, if not just return.
	if (pr==null){
	    return;
	}

	//send the output and err to screen in 2 threads
	Thread stdout = new Thread(new StdOut(pr));
	Thread stderr = new Thread(new StdErr(pr));
	stdout.start();
	stderr.start();
	try{
	    stdout.join();
	    stderr.join();
	}
	catch(InterruptedException e){
	    System.out.println("INTERRUPTED EXCEPTION DETECTED IN OUTPUTING!");
	    e.printStackTrace();
	}
    }
			
}

//Class used to handle standard output
class StdOut implements Runnable{
    Process p;
    
    public StdOut(Process p){
	this.p=p;
    }
    
    public void run(){
	try{
	    BufferedReader br= new BufferedReader(new InputStreamReader(p.getInputStream()));
	    String s;
	    while((s=br.readLine())!=null){
		System.out.println(s);
	    }
	}
	catch(IOException e){
	    System.out.println("IO ERROR IN STDOUT!");
	    e.printStackTrace();
	}
    }
}
	
	
//Class used to handle standard error 
class StdErr implements Runnable{
    Process p;
    
    public StdErr(Process p){
	this.p=p;
    }
    
    public void run(){
	try{
	    BufferedReader br= new BufferedReader(new InputStreamReader(p.getErrorStream()));
	    String s;
	    while((s=br.readLine())!=null){
		System.out.println(s);
	    }
	}
	catch(IOException e){
	    System.out.println("IO ERROR IN STDERR!");
	    e.printStackTrace();
	}
    }
}



