import java.util.*;
import java.io.*;

public class Shell{
    private static boolean interact;//true if in interactive mode
    private static BufferedReader in;//source of input
    private static String verbose;//1 if in verbose mode
    private static String shellpath;//shellpath

    //main method
    public static void main (String[] args) {
	//get environment variable
	verbose = System.getProperty("user.verbose","0");
	shellpath = System.getProperty("user.shellpath","");
	
	//get input source
	switch (args.length){    
	case 0:	//interactive mode
	    in = new BufferedReader (new InputStreamReader(System.in));
	    interact = true;
	    break;
	case 1: //batch mode
	    File inFile = new File(args[0]);
	    if (!inFile.exists()){
		System.out.println("File not found. Exiting....");
		System.exit(1);
	    }
	    else if (!inFile.canRead()){
		System.out.println("File cannot be read. Exiting....");
		System.exit(1);
	    }
	    else if (!inFile.isFile()){
		System.out.println("That is not a file. Exiting....");
		System.exit(1);
	    }
	    try{
		in = new BufferedReader (new FileReader(inFile));
	    } catch (FileNotFoundException e){
		System.out.println("File not found. Exiting....");
		System.exit(1);
	    }
	    interact = false;
	    break;
	default:	
	    System.out.println("Wrong number of arguments. Exiting....");
	    System.exit(1);
	    break;
	}
	while (true){
	    exec(parse());
	}
    }

    //parse input    
    private static String[] parse(){
	String inLine = "";//input line
	String[] args;//parsed arguments
	StringTokenizer token;
	int i = 0;//loop counter
	
	if (interact) System.out.print("#");
	try{
	    inLine = in.readLine();
	}catch (IOException e){
	    System.out.println("I/O Exception occurred. Exiting....");
	    System.exit(1);
	}	
	//Ctrl-D check and exit check	
	if (inLine == null || inLine.equals("exit")) {
	    System.out.print("\n");System.exit(1);}
	//tokenize input
	token = new StringTokenizer (inLine, " ");
	args = new String [token.countTokens()];
	while (token.hasMoreTokens()){
	    args[i] = token.nextToken();
	    i++;
	}
	if (!interact && verbose.equals("1")) 
	    System.out.println("#"+inLine);
	return args;
	
    }
    
    //execute command
    private static void exec(String[] args){
	StringTokenizer token = new StringTokenizer (shellpath,":");
	String path = "";//current path
	boolean absolute = false;//true if absolute path
	String temp = args[0];//temp string to store args[0] when
	                      //we change the content of args[0]
	//absolute path check	
	if (args.length != 0 && 
	   (args[0].charAt(0) == '/' || args[0].charAt(0) == '.')) 
	    absolute = true;
	//path finding and execution
	while (token.hasMoreTokens()){
	    try{
		path = "";
		args[0] = temp;
		if (args.length == 0) return;	
		path = token.nextToken() + "/";
		if (!absolute){
		    temp = args[0];
		    args[0] = path + args[0];
		}
		Runtime r = Runtime.getRuntime();
		Process p = r.exec(args);	
		Thread t1 = new Thread(new Command(p.getInputStream()));
		Thread t2 = new Thread(new Command(p.getErrorStream()));       
		t1.start();
		t2.start();
		try { 
		    t1.join();
		} catch (InterruptedException e){ 
		    System.out.println("Join interrupted"); 
		}	
		try { 
		    t2.join(); 
		} catch (InterruptedException e){ 
		    System.out.println("Join interrupted"); 
		}		
       		return;
	    }catch (IOException e){
		if (absolute){
		    System.out.println(
			     "Command not found or cannot be executed");       
		    return;
		}
	    }
	}
	System.out.println("Command not found or cannot be executed");	
    }
}


