/** CS 537, Fall 2002, Project 2.
 * An energy customer.
 */
public class Customer implements Runnable, Enron {

    // Current customer state:

    /** Total consumed thus far */
    private int[] consumed  = new int[COMMODITIES];

    /** Report on the total amount consumed thus far.
     * @param result a vector of amounts, one for each commodity, allocated
     *               by the caller and filled in by this Customer.
     */
    public synchronized void getConsumption(int[] result) {
        for (int i = 0; i < COMMODITIES; i++)
            result[i] = consumed[i];
    } // getConsumption(int[])

    /** Consume the indicated amounts of resources.
     * @param result a vector of amounts, one for each commodity.
     */
    private synchronized void consume(int[] amount) {
        for (int i = 0; i < COMMODITIES; i++)
            consumed[i] += amount[i];
    } // consume(int[])

    /** Main loop.
     * Repeatedly generate random buy orders to random traders.
     */
    public void run() {
        int[] order = new int[COMMODITIES];
        for (;;) {
            try {
                Thread.sleep(Project2.randInt(500));
            } catch (InterruptedException e) {
                Project2.setVerbose(true);
                Project2.debug("interrupted while sleeping");
                return;
            }

            for (int i = 0; i < order.length; i++) {
                order[i] = Project2.randInt(1,10);
            }
            int whichTrader = Project2.randInt(Project2.COMMODITIES);
            Project2.debug(
                "requesting " + Project2.orderToString(order)
                + " from " + cname[whichTrader] + " trader");
            Project2.specialist(whichTrader).buy(order);
            Project2.debug(
                "got " + Project2.orderToString(order)
                + " from " + cname[whichTrader] + " trader");
            consume(order);
        }
    } // run()
} // Customer
