/** CS 537, Fall 2002, Project 2.
 * Main program.
 */
import java.util.Random;

/** The Project 2 main class. */
public class Project2 implements Enron {

    /** Number of Customers. */
    private static int customerCount;

    /** Array of Trader instances.  trader[i] is a Trader specializing in
     * commodity i.
     */
    private static Trader[] trader = new Trader[COMMODITIES];

    /** Array of Customer instances. */
    private static Customer[] customer;

    /** Array of Customer threads. */
    private static Thread[] cthread;

    /** The unique supplier. */
    private static Supplier supplier;

    /** Flag to control debugging output. */
    private static boolean verbose = false;

    /** Turn debugging output on or off.
     * @param on if true, turn debugging on; otherwise turn it off.
     */
    public static void setVerbose(boolean on) {
        verbose = on;
    } // setVerbose(boolean)

    /** Return the specialist for a given commodity.
     * @param commodity the commodity code.
     * @return the Trader that specializes in this commodity.
     */
    public static Trader specialist(int commodity) {
        return trader[commodity];
    } // specialist(int)

    /** If the debugging flag is on, print a message, preceded by the
     * name of the current thread.  If it is off, do nothing.
     * @param message the message to print.
     */
    public static void debug(Object message) {
        if (verbose)
            System.out.println(Thread.currentThread().getName()
                + ": " + message);
    } // debug(Object)

    /** Turn an order into a printable string.
     * @param order an indication of the amount of each commodity desired.
     */
    public static String orderToString(int[] order) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < order.length; i++) {
            sb.append(i == 0 ? "[" : "; ")
                .append(Integer.toString(order[i]))
                .append(' ')
                .append(cunit[i]);
            if (order[i] != 1) {
                sb.append('s');
            }
            sb.append(' ')
                .append(cname[i]);
        }
        sb.append("]");
        return sb.toString();
    } // orderToString(int[])

    /** Random number generator. */
    private static Random rand;
        
    /** Utility to generate a random integer in the range 0 &lt;= x &lt; max.
     * @param max integer one greater than the largest possible result.
     * @return a non-negative integer strictly less than max.
     */
    public static int randInt(int max) {
        if (0 >= max) {
            throw new IllegalArgumentException("randInt");
        }
        return (rand.nextInt(max));
    } // randInt(int)

    /** Utility to generate a random integer in the range min &lt;= x &lt;= max.
     * @param min the smallest possible result.
     * @param max the largest possible result.
     * @return an integer between min and max, inclusive.
     */
    public static int randInt(int min, int max) {
        if (min >= max) {
            throw new IllegalArgumentException("randInt");
        }
        return min + rand.nextInt(max - min + 1);
    } // randInt(int,int)

    /** Print a usage message and terminate. */
    private static void usage() {
        System.err.println(
            "usage: Project2 [-v][-r] customerCount iterations");
        System.exit(1);
    } // usage()

    /** Main program for project 2.
     * @param args the command-line arguments.
     */
    public static void main(String[] args) {
        // Parse command-line arguments
        GetOpt options = new GetOpt("Project2", args, "vr");
        int c;
        while ((c = options.nextOpt()) != -1) {
            switch (c) {
            default: usage();
            case 'v': verbose = true; break;
            case 'r': rand = new Random(0); break;
            }
        }
        if (rand == null) {
            rand = new Random();
        }
        if (options.optind != args.length - 2) {
            usage();
        }
        customerCount = Integer.parseInt(args[options.optind]);
        int iterations = Integer.parseInt(args[options.optind+1]);

        // Create Traders
        for (int i = 0; i < COMMODITIES; i++) {
            trader[i] = new TraderImplementation(i);
        }

        supplier = new Supplier(iterations);
        Thread pthread = new Thread(supplier, "Supplier");

        customer = new Customer[customerCount];
        cthread = new Thread[customerCount];
        for (int i = 0; i < customerCount; i++) {
            customer[i] = new Customer();
            cthread[i] = new Thread(customer[i], "Customer " + i);
        }

        // Start the threads running
        // They all have lower priority than the main thread so none of them
        // will run until we are done starting them all.
        pthread.setPriority(Thread.NORM_PRIORITY - 1);
        pthread.start();
        for (int i = 0; i < customerCount; i++) {
            cthread[i].setPriority(Thread.NORM_PRIORITY - 1);
            cthread[i].start();
        }

        // Wait for all the threads to finish
        try {
            pthread.join();
            for (int i = 0; i < customerCount; i++) {
                cthread[i].join();
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    } // main(String[])

    /** Gather allocation information from the Supplier, Customers, and Traders
     * and print it.  Then interrupt all the Customer threads.
     */
    public static void terminateProgram() {
        System.out.println("**** Program terminating");

        int[] amt = new int[COMMODITIES];
        int[] balance = new int[COMMODITIES];

        supplier.getProduction(amt);
        System.out.println("Produced " + orderToString(amt));
        for (int i = 0; i < COMMODITIES; i++)
            balance[i] = amt[i];

        for (int i = 0; i < COMMODITIES; i++) {
            trader[i].getAmountOnHand(amt);
            System.out.println("Trader " + i + " has " + orderToString(amt));
            for (int j = 0; j < COMMODITIES; j++)
                balance[j] -= amt[j];
        }

        for (int i = 0; i < customerCount; i++) {
            customer[i].getConsumption(amt);
            System.out.println("Customer " + i
                + " consumed " + orderToString(amt));
            for (int j = 0; j < COMMODITIES; j++)
                balance[j] -= amt[j];
        }
        System.out.println(
            "Net excess (deficit) is " + orderToString(balance));
        System.out.println("*** Interrupting customers");
        for (int i = 0; i < customerCount; i++) {
            cthread[i].interrupt();
        }
    } // terminateProgram()
} // Project2
