import java.util.Random;
import java.text.DecimalFormat;

/** CS 537, Fall 2002, Project 3.
 * An energy customer.
 * <p>Copyright &#169; 2002 by Marvin Solomon.  All rights reserved.</p>
 * @version $Revision: 1.8 $ $Date: 2002/10/17 10:50:05 $
 */
public class Customer implements Runnable, Enron {
    /** Id of this customer */
    private int id;

    /** The trader to buy from. */
    private Trader trader;

    // Parameters that determine the behavior of this customer.

    /** Mean sleep time (in milliseconds) between purchase requests. */
    private int meanSleepTime;

    /** Mean amount of each quantity to purchase on each request. */
    private int[] maxPurchase = new int[COMMODITIES];

    // Current customer state

    /** Total consumed thus far */
    private int[] consumed  = new int[COMMODITIES];

    /** Number of buying requests completed by this Consumer */
    private int numberOfPurchases = 0;

    /** Sum of service times for all requests */
    private int totalServiceTime = 0;

    // Misc fields

    /** A source of random numbers. */
    private Random rand;

    /** A format for printing fractions to two decimal places. */
    private static DecimalFormat fmt = new DecimalFormat("0.00");

    // Constructors

    /** Create a new Customer object.
     * @param id the unique id of this customer.
     * @param trader the trader to contact to buy stuff.
     */
    public Customer(int id, Trader trader) {
        this.id = id;
        this.trader = trader;

        // Initialize the random number generator.  If you use
        //    rand = new Random()
        // you will get a different sequence of random numbers each time you
        // run the program.
        rand = new Random(id * 100);

        // Set values of various simulation parameters.  See also the
        // Supplier constructor.

        // All customers make purchases at the same average rate
        meanSleepTime = 50;

        // All customers generate (roughly) balanced orders, but customer 0 is
        // much more greedy than the others.
        for (int i = 0; i < COMMODITIES; i++) {
            maxPurchase[i] = id == 0 ? 50 : 10;
        }
        
    } // Consumer(Trader)

    // Methods

    /** Abbreviation for System.out.println.
     * @param o the object to be printed.
     */
    private static synchronized void pl(Object o) {
        System.err.println(o);
    } // pl(Object)

    /** Print information about the history of this customer. */
    public synchronized void printReport() {
        String avg = fmt.format(totalServiceTime / (double)numberOfPurchases);
        pl("Customer " + id
            + "\n   Commodities consumed:   " + Project3.orderToString(consumed)
            + "\n   Completed purchases:    " + numberOfPurchases
            + "\n   Total waiting time(ms): " + totalServiceTime
            + "\n   Average waiting time:   " + avg);
    } // printReport()

    /** Main loop.
     * Repeatedly generate random buy orders to the trader until the trader
     * tells us to stop.  Then print some info and return.
     */
    public void run() {
        for (;;) {
            // Sleep for a while.
            int sleepTime = Project3.expo(rand, meanSleepTime);
            try {
                Thread.sleep(sleepTime);
            } catch (InterruptedException e) {
                e.printStackTrace();
                return;
            }

            // Generate an order.  Note the hack to make sure we buy something.
            int[] order = new int[COMMODITIES];
            int sum;
            do {
                sum = 0;
                for (int i = 0; i < order.length; i++) {
                    order[i] = rand.nextInt(maxPurchase[i] + 1);
                    sum += order[i];
                }
            } while (sum == 0);

            int requestTime = Project3.time();
            if (!trader.buy(id, order)) {
                printReport();
                return;
            }
            int  serviceTime = Project3.time() - requestTime;
            totalServiceTime += serviceTime;
            numberOfPurchases++;

            for (int i = 0; i < COMMODITIES; i++) {
                consumed[i] += order[i];
            }
        }
    } // run()
} // Customer
