import java.util.Random;

/** CS 537, Fall 2002, Project 3.
 * Main program.
 * <p>Copyright &#169; 2002 by Marvin Solomon.  All rights reserved.</p>
 * @version $Revision: 1.6 $ $Date: 2002/10/17 10:50:05 $
 */
/** The Project 3 main class. */
public class Project3 implements Enron {
    /** Array of Customer instances. */
    private static Customer[] customer;

    /** Array of Customer threads. */
    private static Thread[] cthread;

    /** The unique Trader. */
    private static Trader trader;

    /** The unique Supplier. */
    private static Supplier supplier;

    /** Startup time (used to support the time() method. */
    static private long startTime = System.currentTimeMillis();

    /** Flag to control debugging output. */
    private static boolean verbose = false;

    /** Number of customers (from the command line). */
    private static int customerCount;

    // Methods

    /** Get the number of customers.
     * @return the number of customers.
     */
    public static int customers() {
        return customerCount;
    } // customers()

    /** Turn debugging output on or off.
     * @param onOff if true, turn debugging on; otherwise turn it off.
     */
    public static synchronized void setVerbose(boolean onOff) {
        verbose = onOff;
    } // setVerbose(boolean)

    /** Debug print.
     * If the debugging flag is on, print a message, preceded by the
     * name of the current thread.  If it is off, do nothing.
     * @param message the message to print.
     */
    public static void debug(Object message) {
        if (verbose)
            System.out.println(Thread.currentThread().getName()
                + ": " + message);
    } // debug(Object)

    /** Handy procedure for timing.
     * @return the elapsed time since startup, in milliseconds.
     */
    static public int time() {
        return (int)(System.currentTimeMillis() - startTime);
    }

    /** Turn an order into a printable string.
     * @param order an indication of the amount of each commodity desired.
     */
    public static String orderToString(int[] order) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < order.length; i++) {
            sb.append(i == 0 ? "[" : ",")
                .append(Integer.toString(order[i]));
        }
        sb.append("]");
        return sb.toString();
    } // orderToString(int[])

    /** Generate an exponentially distributed random number.
     * @param rand the source of random numbers.
     * @param mean the mean of the distribution.
     * @return the next sample value.
     */
    public static int expo(Random rand, int mean) {
        return (int) Math.round(-Math.log(rand.nextDouble()) * mean);
    } // expo(double)

    /** Print a usage message and terminate. */
    private static void usage() {
        System.err.println(
            "usage: Project3 [-v] algorithm customerCount iterations");
        System.exit(1);
    } // usage()

    /** Main program for project 3.
     * @param args the command-line arguments.
     */
    public static void main(String[] args) {
        // Parse command-line arguments
        GetOpt options = new GetOpt("Project3", args, "v");
        int c;

        while ((c = options.nextOpt()) != -1) {
            switch (c) {
            default: usage();
            case 'v': verbose = true; break;
            }
        }
        if (options.optind != args.length - 3) {
            usage();
        }

        int algorithm = Integer.parseInt(args[options.optind+0]);
        customerCount = Integer.parseInt(args[options.optind+1]);
        int iterations = Integer.parseInt(args[options.optind+2]);

        // Create the trader
        trader = new Trader(algorithm);
        Thread tthread = new Thread(trader, "Trader");

        // Create the supplier
        supplier = new Supplier(trader, iterations);
        Thread sthread = new Thread(supplier, "Supplier");

        // Create the customers
        customer = new Customer[customerCount];
        cthread = new Thread[customerCount];
        for (int i = 0; i < customerCount; i++) {
            customer[i] = new Customer(i, trader);
            cthread[i] = new Thread(customer[i], "Customer " + i);
        }

        // Start the threads running
        // They all have lower priority than the main thread so none of them
        // will run until we are done starting them all.
        tthread.setPriority(Thread.NORM_PRIORITY - 1);
        tthread.start();
        sthread.setPriority(Thread.NORM_PRIORITY - 1);
        sthread.start();
        for (int i = 0; i < customerCount; i++) {
            cthread[i].setPriority(Thread.NORM_PRIORITY - 1);
            cthread[i].start();
        }

        // Wait for all the threads to finish
        try {
            tthread.join();
            sthread.join();
            for (int i = 0; i < customerCount; i++) {
                cthread[i].join();
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    } // main(String[])
} // Project3
