import java.util.Random;

/** CS 537, Fall 2002, Project 3.
 * A supplier of energy resources.
 * <p>Copyright &#169; 2002 by Marvin Solomon.  All rights reserved.</p>
 * @version $Revision: 1.9 $ $Date: 2002/10/17 10:45:30 $
 */
public class Supplier implements Runnable, Enron {
    /** The trader to supply. */
    private Trader trader;

    // Parameters that determine the behavior of this customer.

    /** Number of times to iterate before terminating. */
    private int iterations;

    /** Mean time to sleep between iterations (in milliseconds). */
    private int meanSleepTime;

    /** Maximum amount of each commodity to supply on each iteration. */
    private int[] maxSupply = new int[COMMODITIES];

    // Misc fields

    /** Source of random numbers. */
    private Random rand;

    /** Amount supplied. */
    private int[] supplied = new int[COMMODITIES];

    // Constructors

    /** Creates a new supplier.
     * @param trader the trader to supply.
     * @param iteration the number of times to iterate before terminating.
     */
    public Supplier(Trader trader, int iterations) {
        this.trader = trader;
        this.iterations = iterations;

        // Initialize the random number generator.  If you use
        //    rand = new Random()
        // you will get a different sequence of random numbers each time you
        // run the program.
        rand = new Random(99);

        // Set values of various simulation parameters.
        // NOTE:  These numbers are designed so that average rate of production
        // exactly matches the average rate of consumption, as set in the
        // Customer constructor:
        //
        // Customer 0 generates an average of 1/50 order/ms with an average
        // request of 25 units/order of each commodity for an average
        // consumption rate of 25/50 = 0.50 unit/ms.  Similarly, all other
        // customers consume at an average rate of 5/50 = 0.10 unit/ms.
        // So the total average consumption rate is 0.50 + (N-1)*0.10, where
        // N is the number of customers.
        //
        // We set up the supplier to supply 5 units for each iteration,
        // so if S is meanSleepTime and A = maxSupply/2 is the mean amount
        // supplied, we have the equation
        //     A / S = 0.50 + (N-1)*0.10
        // or
        //     S = A / (0.50 + (N-1)*0.10)

        for (int i = 0; i < maxSupply.length; i++) {
            maxSupply[i] = 10;
        }

        meanSleepTime
            = (int) Math.round(5 / (0.50 + (Project3.customers() - 1) * 0.10));

        Project3.debug(
            "Supplying an average of 5 units of each commodity every "
            + meanSleepTime + " ms");
    } // Supplier(Trader,int)

    // Methods

    /** Main loop.
     * Repeatedly supply one unit of each resource to the trader.  After
     * <i>iterations</i> iterations, tell the trader to shut down.
     */
    public void run() {
        int[] amount = new int[COMMODITIES];
        for (int i = 0; i < iterations; i++) {
            try {
                int slp = Project3.expo(rand, meanSleepTime);
                Thread.sleep(slp);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            for (int j = 0; j < amount.length; j++) {
                amount[j] = rand.nextInt(maxSupply[j] + 1);
            }
            trader.supply(amount);
            for (int j = 0; j < supplied.length; j++) {
                supplied[j] += amount[j];
            }
        }
        pl("Supplier");
        pl("   Iterations:     " + iterations);
        pl("   Total supplied: " + Project3.orderToString(supplied));
        trader.shutDown();
    } // run()

    /** Abbreviation for System.out.println.
     * @param o the object to be printed.
     */
    private static void pl(Object o) {
        System.err.println(o);
    } // pl(Object)
} // Supplier
