// the DiskTester class

public class DiskTester {

    public static void main(String args[]) {
	long startT = Library.getTime();
        Library.output("DiskTester launched. Time = "+startT+"\n");
        if (args.length < 1) {
            Library.output("Tests the disk for errors and performance.\n");
            Library.output("Type DiskTester <operation> for details.\n");
            Library.output("Currently supported operations:\n");
            Library.output(" random \n");
            Library.output(" surface \n");
            return;
        }
    opcode_switch:
        {
            if (args[0].equals("random")) {
                if (args.length != 2) {
                    Library.output("Performs random pattern tests on disks.\n");
                    Library.output("Usage: DiskTester random <count>\n");
                    break opcode_switch;
                }        
                int count = Integer.parseInt(args[1]);
                random(count);
            } else if (args[0].equals("surface")) {
                if (args.length != 2) {
                    Library.output("Performs surface scan on all disk blocks.\n");
                    Library.output("Usage: DiskTester surface <tests_per_block>\n");
                    break opcode_switch;
                }
                int count = Integer.parseInt(args[1]);
                surfaceScan(count);
            }
        } // end opcode_switch

        Library.output("DiskTester stopped. Duration Time = "+ (Library.getTime() - startT)+"\n");
    }
    public static void random(int count) {
        int blockSize = Library.getDiskBlockSize();
        int numBlocks = Library.getDiskBlockCount();
        byte[] buffer = new byte[blockSize];        

        for (int i = 0; i < count; i++) {
            int j = (int) (Math.random() * numBlocks); // block j
            int k = (int) (Math.random() * 256);      // pattern k

            // test for the read and write to disk
            setPattern(k, buffer);
            Library.writeDiskBlock(j, buffer);
            clearPattern(buffer);
            Library.readDiskBlock(j, buffer);
            if (!checkPattern(k, buffer)) {
                Library.output("Test block "+j+", pattern "+k+" failed! \n");
            } else {
                Library.output("Test block "+j+", pattern "+k+" passed! \n");
            }
        }
    }
    
    public static void surfaceScan(int count) {
        int blockSize = Library.getDiskBlockSize();
        int numBlocks = Library.getDiskBlockCount();
        byte[] buffer = new byte[blockSize];        
        boolean success = true;
        for (int i = 0; i < count; i++) {
            // write to all disk blocks
            for (int j = 0; j < numBlocks; j++) {   // block j
                setPattern(i+j, buffer);
                Library.writeDiskBlock(j, buffer);
            }
            clearPattern(buffer);
            // read back and check all disk blocks
            for (int j = 0; j < numBlocks; j++) {
                Library.readDiskBlock(j, buffer);
                if (!checkPattern(i+j, buffer)) {
                    Library.output("Surface scan failed at block "+j+"\n");
                    success = false;
                }
            }
        }
        // finished checking all blocks. Print result summary
        if (success) {
            Library.output("Surface scan complete. No errors detected.\n");
        }
    }

    private static void setPattern(int blockNumber, byte[] buffer){
        for (int i=0; i < buffer.length; i++){
            buffer[i] = (byte) (blockNumber + i);
        }
    }

    private static boolean checkPattern(int blockNumber, byte[] buffer){
        for (int i = 0; i <buffer.length; i++){
            if (buffer[i] != (byte) (blockNumber + i))
                return false;
        }
        return true;
    }

    private static void clearPattern(byte[] buffer) {
        for (int i=0; i <buffer.length; i++) {
            buffer[i]=0;
        }
    }
}
