/**
 *   Each disk access request will be handled by an instance of this class. 
 *   Classes that wait for the completion of this request can call the
 *   waitForCompletion() method and they will be awaken when it is finished.
 *
 */
public class Request {

    /** Two state variables: 
     *  requestType:    Type of request. (0-2)
     *  status:         how the request is doing now (0-5)
     */

    // 
    public static final int INVALID=0;

    //requestType
    public static final int READ=1;
    public static final int WRITE=2;

    //status
    public static final int REQUESTED=1;
    public static final int SCHEDULED=2;
    public static final int PROCESSING=3;
    public static final int COMPLETED=4;
    public static final int ABORTED=5;
    
    public int requestType;
    public int status;
    public int blockNumber;
    public byte[] buffer;


    // Disable default constructor.
    private Request() {}

    public Request(int requestType,
				int blockNumber,
				byte[] buffer) {
	switch (requestType) {
	case READ:
	    this.requestType = requestType;
	    this.blockNumber = blockNumber;
	    this.buffer = buffer;
	    this.status = REQUESTED;
	    break;
	case WRITE:
	    this.requestType = requestType;
	    this.blockNumber = blockNumber;
	    this.buffer = buffer;
	    this.status = REQUESTED;
	    break;
	default:
	    this.requestType = INVALID;
	    this.blockNumber = -1;
	    this.buffer = null;
	    this.status = INVALID;
	    break;	    
	}
    }

    /*
    public synchronized void signalActivation() {
	if (status == INVALID) return;
	status = PROCESSING;
	notify();
    }
    */

    public synchronized void signalCompletion() {
	if (status == INVALID) return;
	status = COMPLETED;
	notify();
    }

    /*
    public synchronized void signalAbortion() {
	if (status == INVALID) return;
	status = ABORTED;
	notify();
    }
    */

    public synchronized void waitForCompletion() {
	if (status == INVALID) return;
	while (status == REQUESTED || 
	       status == SCHEDULED || 
	       status == PROCESSING) { 
	    try {
		wait();
	    } catch (InterruptedException ex) {
		ex.printStackTrace();
	    }
	}
    }
    
    public synchronized String toString() {
	StringBuffer s=new StringBuffer("(Request) ");
	switch (requestType) {
	case READ:
	    s.append("READ block="+blockNumber);	    
	    break;
	case WRITE:
	    s.append("WRITE block="+blockNumber);
	    break;
	default:
	    s.append("INVALID ");
	    break;
	}
	return s.toString();
    }
}



