import java.util.*;

public class Scheduler {

    protected boolean available;

    // List of scheduled requests. 
    // Requests that are currently being processed
    //     are removed from this list before commencing. 
    // Requests that are started immediately without waiting 
    //     are not added to this list at all.
    // 
    protected Vector requestList;
    protected Disk disk;
    
    protected Request currentRequest;

    // Disable default constructor
    protected Scheduler() {}

    public Scheduler(int cacheSize, Disk disk) {
	this.disk = disk;
	requestList = new Vector();
	available = true;
    }

    public synchronized Request requestRead(int blockNumber, byte[] data) {
	Request req = new Request(Request.READ, blockNumber, data);
	if (available) {
	    available = false;
	    currentRequest = req;
	    //	    req.signalActivation();
	    System.out.println("Scheduler: physical read "+blockNumber);
	    disk.beginRead(blockNumber, data);
	} else {
	    requestList.add(req);
	}
	return req;
    }

    public synchronized Request requestWrite(int blockNumber, byte[] data) {
	Request req = new Request(Request.WRITE, blockNumber, data);
	if (available) {
	    available = false;
	    currentRequest = req;
	    //      req.signalActivation();
	    System.out.println("Scheduler: physical write "+blockNumber);
	    disk.beginWrite(blockNumber, data);
	} else {
	    requestList.add(req); 
	}
	return req;
    }

    public synchronized void interrupt() {
	currentRequest.signalCompletion();
	int lastReqBlockNumber = currentRequest.blockNumber;
	currentRequest = null;
	if (requestList.size()==0) {
	    available = true;
	} else {
	    // This defines a FCFS
	    // currentRequest = (Request) requestList.remove(0); 
	    // Elevator alg
	    {
		int nextFwdBlockNumber = disk.DISK_SIZE+1;
		int nextBackBlockNumber = lastReqBlockNumber+1;
		int nextFwdIndex = -1;
		int nextBackIndex = -1;
		for (int i = 0; i < requestList.size(); i++) {
		    Request r = (Request) requestList.get(i);
		    if (r.blockNumber > lastReqBlockNumber) {
			// This item is located after the current position
			if (r.blockNumber < nextFwdBlockNumber) {
			    nextFwdBlockNumber = r.blockNumber;
			    nextFwdIndex = i;
			}
		    } else {
			// This item is closer to the beginning of disk
			if (r.blockNumber < nextBackBlockNumber) {
			    nextBackBlockNumber = r.blockNumber;
			    nextBackIndex = i;
			}
		    }
		}
		if (nextFwdIndex != -1) {
		    currentRequest = (Request) requestList.remove(nextFwdIndex);
		} else if (nextBackIndex != -1) {
		    currentRequest = (Request) requestList.remove(nextBackIndex);
		} else {
		    // unreachable, since this implies requestList.size() == 0 
		    available = true;
		    System.out.println("Elevator Alg error in Scheduler.");
		}
	    }
	    
	    
	    
	    //currentRequest.signalActivation();
	    switch (currentRequest.requestType) {
	    case Request.READ:
		disk.beginRead(currentRequest.blockNumber, 
			       currentRequest.buffer);
		System.out.println("Scheduler: physical read "+
				   currentRequest.blockNumber);
		break;
	    case Request.WRITE:
		disk.beginWrite(currentRequest.blockNumber,
				currentRequest.buffer);
		System.out.println("Scheduler: physical write "+
				   currentRequest.blockNumber);
		break;
	    default:
		break;
	    }
	}	
    }

    public void shutdown() {
    }
    

}




