/**
 *  The CacheBlock Class implement an element inside the cache
 *  By its status, the Buffered Scheduler will imitate functioning  
 *  with the disk with the Cache (a queue of CacheBlock)
 */
public class CacheBlock {

    /** state variables: 
     *  status: what is the state of the cacheblock (0-4)
     */

    // 
    public static final int UNUSED=0;
    public static final int READING=1;
    public static final int WRITING=2;
    public static final int DIRTY=3;
    public static final int CLEAN=4;
    
    public int status;
    public int blockNumber;
    public byte[] buffer;
    public int waitCount;

    // Disable default constructor.
    private CacheBlock() {}

    public CacheBlock(byte[] buffer) {
	status = UNUSED;
	blockNumber = -1; //initialized as an invalid blockNumber
	this.buffer = buffer;
	waitCount = 0;
    }

    public synchronized void signalCompletion() {
	//	if (status == CLEAN){ 
	notifyAll();
	//}
    }

    public synchronized void waitForCompletion() {
	while (status == READING || 
	       status == WRITING || 
	       status == UNUSED) { 
	    try {
		wait();
	    } catch (InterruptedException ex) {
		System.out.println("Inside CacheBlock (blocknumber) "
				   + blockNumber +
				   ". Method: waitForCompletion" );
		ex.printStackTrace();
	    }
	}
    }
    
    public synchronized String toString() {
	StringBuffer s=new StringBuffer("(CacheBlock) ");
	switch (status) {
	case READING:
	    s.append("READING block = "+blockNumber);	    
	    break;
	case WRITING:
	    s.append("WRITING block = "+blockNumber);
	    break;
	case DIRTY:
	    s.append("DIRTY block = "+blockNumber);
	    break;
	case CLEAN:
	    s.append("CLEAN block = "+blockNumber);
	    break;
	default:
	    s.append("UNUSED");
	    break;
	}
	return s.toString();
    }
}
