// the DiskTester class

public class DiskTester {

    public static long startTime;

    public static long testStartTime, testStopTime;

    public static void main(String args[]) {
	startTime = Library.getTime();
	if (args.length < 1) {
	    Library.output("Tests the disk for errors and performance.\n");
	    Library.output("Type DiskTester <operation> for details.\n");
	    Library.output("Currently supported operations:\n");
	    Library.output(" random      Performs random pattern tests on disk \n");
	    Library.output(" surface     Scans the whole disk for defects \n");
	    Library.output(" ranged      Performs random access within a range \n");
	    Library.output(" sequential  Performs sequential access within a range \n");
	    return;
	}
	Library.output("DiskTester launched. Starting Time="+startTime+"\n"); 
    opcode_switch:
	{
	    if (args[0].equals("random")) {
		if (args.length != 2) {
		    Library.output("Performs random pattern tests on disks.\n");
		    Library.output("Usage: DiskTester random <count>\n");
		    break opcode_switch;
		}	
		int count = Integer.parseInt(args[1]);
		testStartTime = Library.getTime();
		random(count);
		testStopTime = Library.getTime();
		long elapsedTime = (testStopTime - testStartTime);
		Library.output("Random Pattern test finished. Statistics:\n");
		Library.output("Total time elapsed:               "+elapsedTime+"\n");
		Library.output("Average time per Read-Write pair: "+(((double)elapsedTime)/count)+"\n");
	    } else if (args[0].equals("surface")) {
		if (args.length != 2) {
		    Library.output("Performs surface scan on all disk blocks, in order to\n");
		    Library.output("verify total disk capacity and report any individual block defect.\n");
		    Library.output("Usage: DiskTester surface <tests_per_block>\n");
		    break opcode_switch;
		}
		int count = Integer.parseInt(args[1]);
		testStartTime = Library.getTime();
		surfaceScan(count);
		testStopTime = Library.getTime();
		long elapsedTime = (testStopTime - testStartTime);
		Library.output("Surface Scan finished. Statistics:\n");
		Library.output("Total time elapsed:               "+elapsedTime+"\n");
	    } else if (args[0].equals("ranged")) {
		if (args.length != 4) {
		    Library.output("Performs random read/writes on blocks within a specified range.\n");
		    Library.output("Usage: DiskTester ranged <num_test> <start_block> <stop_block>\n");
		    break opcode_switch;
		}
		int count = Integer.parseInt(args[1]);
		int rangeStart = Integer.parseInt(args[2]);
		int rangeStop = Integer.parseInt(args[3]);
		testStartTime = Library.getTime();
		int accessCounter = ranged(count, rangeStart, rangeStop);
		testStopTime = Library.getTime();
		long elapsedTime = (testStopTime - testStartTime);
		Library.output("Random Test finished. Statistics:\n");
		Library.output("Total time elapsed:               "+elapsedTime+"\n");
		Library.output("Average time per access:          "+
			       (((double)elapsedTime)/accessCounter)+"\n");   
	    } else if (args[0].equals("sequential")) {
		if (args.length != 4){
		    Library.output("Performs sequential accesses within a range.\n");
		    Library.output("For example, (1,4) will generate R1 R2 R3 R4 [R1 R2 R3 R4 [...]] \n");
		    Library.output("Usage: DiskTester sequential <num_rounds> <start_block> <stop_block>\n");
		    break opcode_switch;
		}
		int count = Integer.parseInt(args[1]);
		int rangeStart = Integer.parseInt(args[2]);
		int rangeStop = Integer.parseInt(args[3]);
		testStartTime = Library.getTime();
		int accessCounter = sequential(count, rangeStart, rangeStop);
		testStopTime = Library.getTime();
		long elapsedTime = (testStopTime - testStartTime);
		Library.output("Sequential test finished. Statistics:\n");
		Library.output("Total time elapsed:               "+elapsedTime+"\n");
		Library.output("Average time per access:          "+
			       (((double)elapsedTime)/accessCounter)+"\n");   
	    }
	} // end opcode_switch

	Library.output("DiskTester stopped. Time="+Library.getTime()+"\n");
    }
    public static void random(int count) { 
	int blockSize = Library.getDiskBlockSize();
	int numBlocks = Library.getDiskBlockCount();
	byte[] buffer = new byte[blockSize];	

	for (int i = 0; i < count; i++) {
	    int j = (int) (Math.random() * numBlocks); // block j
	    int k = (int) (Math.random() * 256);      // pattern k

	    // test for the read and write to disk
	    setPattern(k, buffer);
	    Library.writeDiskBlock(j, buffer);
	    clearPattern(buffer);
	    Library.readDiskBlock(j, buffer);
	    if (!checkPattern(k, buffer)) {
		Library.output("Test block "+j+", pattern "+k+" failed! \n");
	    } else {
		Library.output("Test block "+j+", pattern "+k+" passed! \n");
	    }
	}
    }
    
    public static void surfaceScan(int count) {
	int blockSize = Library.getDiskBlockSize();
	int numBlocks = Library.getDiskBlockCount();
	byte[] buffer = new byte[blockSize];	
	boolean success = true;
	for (int i = 0; i < count; i++) {
	    // write to all disk blocks
	    for (int j = 0; j < numBlocks; j++) {   // block j
		setPattern(i+j, buffer);
		Library.writeDiskBlock(j, buffer);
	    }
	    clearPattern(buffer);
	    // read back and check all disk blocks
	    for (int j = 0; j < numBlocks; j++) {
		Library.readDiskBlock(j, buffer);
		if (!checkPattern(i+j, buffer)) {
		    Library.output("Surface scan failed at block "+j+"\n");
		    success = false;
		}
	    }
	}
	// finished checking all blocks. Print result summary
	if (success) {
	    Library.output("Surface scan complete. No errors detected.\n");
	}
    }

    public static int ranged(int count, int start, int stop) {
	int range=start - stop + 1;
	int event=0;	
	int blockSize = Library.getDiskBlockSize();
	int numBlocks = Library.getDiskBlockCount();
	byte[] buffer = new byte[blockSize];	
	setPattern(range, buffer);   // In this test, pattern is not important.
	for (int i=0; i<count; i++) {
	    int blockNumber = start + (int) (Math.random() * range);
	    // generate either read or write, each 50%
	    if (Math.random() > 0.5) {
		Library.readDiskBlock(blockNumber, buffer);
	    } else {
		Library.writeDiskBlock(blockNumber, buffer);
	    }
	    event++;
	}
	return event;
    }

    public static int sequential(int count, int start, int stop) {
	int event=0;
	int blockSize = Library.getDiskBlockSize();
	int numBlocks = Library.getDiskBlockCount();
	byte[] buffer = new byte[blockSize];	
	setPattern(count, buffer);   // In this test, pattern is not important.
	for (int c=0; c<count; c++) {
	    for (int b=start; b<=stop; b++) {
		Library.readDiskBlock(b, buffer);
		event++;
	    } 
	}
	return event;
    }

    private static void setPattern(int blockNumber, byte[] buffer){
	for (int i=0; i < buffer.length; i++){
	    buffer[i] = (byte) (blockNumber + i);
	}
    }

    private static boolean checkPattern(int blockNumber, byte[] buffer){
	for (int i = 0; i <buffer.length; i++){
	    if (buffer[i] != (byte) (blockNumber + i))
		return false;
	}
	return true;
    }

    private static void clearPattern(byte[] buffer) {
	for (int i=0; i <buffer.length; i++) {
	    buffer[i]=0;
	}
    }
}






