/**
 * Request from the user
 */
public class UserRequest {

    // 
    public static final int INVALID=0;

    //requestType
    public static final int READ=1;
    public static final int WRITE=2;

    //status
    public static final int REQUESTED=1;
    public static final int SCHEDULED=2;
    public static final int PROCESSING=3;
    public static final int COMPLETED=4;
    public static final int ABORTED=5;
    
    public int requestType;
    public int status;
    public int blockNumber;
    public byte[] buffer;

    public int cacheAllocated;

    // Disable default constructor.
    private UserRequest() {}

    public UserRequest(int requestType,
				int blockNumber,
				byte[] buffer) {
	switch (requestType) {
	case READ:
	    this.requestType = requestType;
	    this.blockNumber = blockNumber;
	    this.buffer = buffer;
	    this.status = REQUESTED;
	    this.cacheAllocated = -1;
	    break;
	case WRITE:
	    this.requestType = requestType;
	    this.blockNumber = blockNumber;
	    this.buffer = buffer;
	    this.status = REQUESTED;
	    this.cacheAllocated = -1;
	    break;
	default:
	    this.requestType = INVALID;
	    this.blockNumber = -1;
	    this.buffer = null;
	    this.status = INVALID;
	    break;	    
	}
    }

    /*
    public synchronized void signalActivation() {
	if (status == INVALID) return;
	status = PROCESSING;
	notify();
    }
    */

    public synchronized void signalCompletion() {
	//if (status == INVALID) return;
	status = COMPLETED;
	notifyAll();
    }

    /*
    public synchronized void signalAbortion() {
	if (status == INVALID) return;
	status = ABORTED;
	notify();
    }
    */

    public synchronized void waitForCompletion() {
	while (status != COMPLETED) { 
	    try {
		wait();
	    } catch (InterruptedException ex) {
		System.out.println("Insider UserRequest wait for completion");
		ex.printStackTrace();
	    }
	}
    }
    
    public synchronized void signalCacheFree(int cacheNumber) {
	if (status == INVALID) return;
	cacheAllocated = cacheNumber;
	notify();
    }

    public synchronized void waitForCacheFree() {
	if (status == INVALID) return;
	while (cacheAllocated == -1) { 
	    try {
		wait();
	    } catch (InterruptedException ex) {
		System.out.println("Insider UserRequest wait for Cache space");
		ex.printStackTrace();
	    }
	}
//	cacheAllocated = false; // so that signalCompletion won't wake it
    }
    
    public synchronized String toString() {
	StringBuffer s=new StringBuffer("(UserRequest) ");
	switch (requestType) {
	case READ:
	    s.append("READ block="+blockNumber);	    
	    break;
	case WRITE:
	    s.append("WRITE block="+blockNumber);
	    break;
	default:
	    s.append("INVALID ");
	    break;
	}
	return s.toString();
    }
}





