public class c6_delete extends FileSystem {


    /***
     * delete the file indicate by the filenumber
     * Find the inode correspond to the file, deallocate all the data and
     * indirect blocks point by the inode by marking "free" (0) in the free
     * map. Moreover, the inode is marked "unused" by setting length = -1
     *
     * NOTE: The inode block or indirect block pointing to the inode is not
     * deallocated.
     *
     * @param fileNumber the inumber of the file being delete
     * @return int -1 for the file being delete is not exist
     *
     */
    public int delete(int fileNumber) {
	if (fileNumber < 0 || fileNumber >= maxFiles) {
	    System.out.println("delete error: invalid fileNumber ("+
			       fileNumber +")");
	    return -1;
	}
	
	// locate the inode
	Inode inode = namei(fileNumber, READ);
	if (inode == null) { // file does not exist
	    return -1;
	}
	
	// access each data block of the file to free it
	int numBlk = (inode.length / blockSize) +
	    ( ((inode.length % blockSize) == 0) ? (0) : (1) );
	for (int i=0; i<=numBlk; i++) {
	    int blkNum = getBlockNum(READ, inode, i);
	    if (blkNum == -1) {
		System.out.println("delete: getBlockNum: error");
		return -1;
	    }
	    else {
		// check if the block is allocated, if yes, free it
		checkNFreeBlk(blkNum);
	    }
	}
	
	// free the indirect blocks inside

	// deallocate indirect1
	checkNFreeBlk(inode.indirect1); // single lvl

	// deallocate all the block pointing from indirect2
	if (inode.indirect2 !=0) { // 2 lvl
	    // iterate through all the pointer in indirect2
	    for (int i=0; i<ptrPerBlock; i++){
		int tmpBlkNum = dereference(READ, inode.indirect2, i);
		checkNFreeBlk(tmpBlkNum);
	    }
	    // deallcated indirect2 itself
	    checkNFreeBlk(inode.indirect2);
	}
	
	// deallcate all the block pointing from indirect3
	if (inode.indirect3 !=0) { // 3 lvl
	    for (int i=0; i<ptrPerBlock;i++){
		int tmpBlkNum2 = dereference(READ, inode.indirect3, i);
		for (int j=0; i<ptrPerBlock; j++){
		    int tmpBlkNum1 = dereference(READ, tmpBlkNum2, j);
		    checkNFreeBlk(tmpBlkNum1);
		}
		checkNFreeBlk(tmpBlkNum2);
	    }
	    checkNFreeBlk(inode.indirect3);
	}

	// mark inode as "unused"
	inode.length = -1;
	
	// update the state of this inode in the InodeBlock
	byte[] buffer = new byte[blockSize];
	
	disk.read(inode.inodeBlock, buffer);
	// Save this inode record into the Inode-block.
	inode.pack(buffer, inode.inodeBlockOffset);
	disk.write(inode.inodeBlock, buffer);
	System.out.println("block["+inode.inodeBlock+"].inode["+
			   inode.inodeBlockOffset+"]= (inumber="+
			   inode.inumber+"; size="+
			   inode.length+")");	
	
	// finish delete , return 
	return 0;
    }

    public void checkNFreeBlk(int pointer){
	if (pointer == -1){
	    System.out.println("Some Error Occur in checkNFreeBlk(" 
			       + pointer+ ")" );
	}
	if (pointer != 0){
	    int checkError = releaseBlock(pointer);
	    if (checkError == -1) {
		System.out.println("checkNFreeBlk: Cannot release block ("+
				   pointer +")");
	    }
	}
    }

    public int size(int fileNumber){
	if (fileNumber < 0 || fileNumber >= maxFiles) {
	    System.out.println("delete error: invalid fileNumber ("+
			       fileNumber +")");
	    return -1;
	}
	
	// locate the inode
	Inode inode = namei(fileNumber, READ);
	if (inode == null) { // file does not exist
	    return -1;
	}

	return inode.length;
    }
}
